//
// i2c_sht75.cpp - Sensirion SHT75 sensor sample
// compile: gcc i2c_sht75.c -o i2c_sht75 -l iowkit
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <math.h>


#include "iowkit.h"

#define FALSE 0
#define TRUE 1

#define HUMIDITY 0x05
#define TEMPERATURE 0x03


void CalcTRH(float *humidity, float *temperature)
{
	const float C1 = -4.0;              // for 12 Bit
	const float C2 = +0.0405;           // for 12 Bit
	const float C3 = -0.0000028;        // for 12 Bit
	const float T1 = +0.01;             // for 14 Bit @ 5V
	const float T2 = +0.00008;           // for 14 Bit @ 5V	

	float rh = *humidity;					// rh:      Humidity [Ticks] 12 Bit 
	float t = *temperature;				// t:       Temperature [Ticks] 14 Bit
	float rh_lin;							// rh_lin:  Humidity linear
	float rh_true;						// rh_true: Temperature compensated humidity
	float t_C;							// t_C   :  Temperature [°C]

	t_C = t*0.01 - 40;						//calc. temperature from ticks to [°C]
	rh_lin = C3*rh*rh + C2*rh + C1;			//calc. humidity from ticks to [%RH]
	rh_true = (t_C-25)*(T1+T2*rh)+rh_lin;		//calc. temperature compensated humidity [%RH]

	if(rh_true>100) rh_true = 100;		//cut if the value is outside of
	if(rh_true<0.1) rh_true = 0.1;		//the physical possible range


	*temperature = t_C;		//return temperature [°C]
	*humidity = rh_true;		//return humidity[%RH]
}

short ReadI2C(IOWKIT_HANDLE handle, UCHAR reg)
{
	IOWKIT_SPECIAL_REPORT report;
	memset(&report, 0x00, IOWKIT_SPECIAL_REPORT_SIZE);
	short result = 0;

	report.ReportID = 0x03;			//I2C-Read
	report.Bytes[0] = 0x03;			//Read 3 Bytes
	report.Bytes[1] = reg;		//Address for humidity or temperature

	IowKitWrite(handle, IOW_PIPE_SPECIAL_MODE, (char*) &report, IOWKIT_SPECIAL_REPORT_SIZE);
	IowKitRead(handle, IOW_PIPE_SPECIAL_MODE, (char*) &report, IOWKIT_SPECIAL_REPORT_SIZE);

	result = (report.Bytes[1] << 8) | report.Bytes[2];	//create a 16bit value

	return result;

}

int main(int argc, char* argv[]) 
{
	unsigned short sn[9];
	char snt[9];
	int j;

	IOWKIT_HANDLE ioHandle = IowKitOpenDevice();	//Get first device handle which found

	if(ioHandle != NULL)
	{
		//Get serial number of device and output
		IowKitGetSerialNumber(ioHandle, sn);

		for (j = 0; j < 9; j++)
			snt[j] = sn[j];

		printf("S/N: %s\n", snt);


		//Enable I2C Mode and set sensibus ON
		IOWKIT_SPECIAL_REPORT report;
		memset(&report, 0x00, IOWKIT_SPECIAL_REPORT_SIZE);

		report.ReportID = 0x01; //I2C-Mode
		report.Bytes[0] = 0x01; //Enable
		report.Bytes[1] = 0xC0; //Disable Pull-Up resistors, Enable Bus
		IowKitWrite(ioHandle, IOW_PIPE_SPECIAL_MODE, (char*) &report, IOWKIT_SPECIAL_REPORT_SIZE);
				
		//Read data
		float humidity = ReadI2C(ioHandle, HUMIDITY);				//Read the humidity over I2C
		float temperature = ReadI2C(ioHandle, TEMPERATURE);		//Read the temperature over I2C

		CalcTRH(&humidity, &temperature);				//calc humidity and temperature
		
		printf("Temperature: %0.2f Celsius\n", temperature);
		printf("Humidity: %0.2f %%\n", humidity);
		
		
		//Disable I2C at befor quit the application
		memset(&report, 0x00, IOWKIT_SPECIAL_REPORT_SIZE);
		report.ReportID = 0x01; //I2C-Mode
		report.Bytes[0] = 0x00; //Disabl
		IowKitWrite(ioHandle, IOW_PIPE_SPECIAL_MODE, (char*) &report, IOWKIT_SPECIAL_REPORT_SIZE);
		
	}
	else
		printf("No device found !");


	IowKitCloseDevice(ioHandle);	//Don't forget to close if you close the application !

	return 0;
}
